@@ -53,15 +53,11 @@ function logdie ($logger, $str)
 	die($str . "\n");
 }
 
-function getHttpVal ($key, $default)
+function getHttpVal ($key, $default, $a)
 {
-	if (array_key_exists($key, $_GET))
+	if (array_key_exists($key, $a))
 	{
-		$val = $_GET[$key];
-	}
-	elseif (array_key_exists($key, $_POST))
-	{
-		$val = $_POST[$key];
+		$val = $a[$key];
 	}
 	else
 	{
@@ -150,7 +146,7 @@ function retrieveURLasync($ident, $urls, $logger, $ans_req=1, $match="^OK", $ret
 		$handle = curl_init();
 		curl_settings($logger, $ident, $handle, $url, $timeout, $curlopts);
 		curl_multi_add_handle($mh, $handle);
-		$ch[$handle] = $handle;
+		$ch[(int) $handle] = $handle;
 	}
 
 	$ans_arr = array();
@@ -197,7 +193,7 @@ function retrieveURLasync($ident, $urls, $logger, $ans_req=1, $match="^OK", $ret
 
 				curl_multi_remove_handle($mh, $info['handle']);
 				curl_close($info['handle']);
-				unset($ch[$info['handle']]);
+				unset($ch[(int) $info['handle']]);
 			}
 
 			curl_multi_select($mh);
@@ -246,6 +242,9 @@ function KSMdecryptOTP($urls, $logger, $curlopts)
 
 function sendResp($status, $logger, $apiKey = '', $extra = null)
 {
+	if ($logger->request !== NULL)
+		$logger->request->set('status', $status);
+
 	$a['status'] = $status;
 
 	// 2008-11-21T06:11:55Z0711
@@ -273,6 +272,9 @@ function sendResp($status, $logger, $apiKey = '', $extra = null)
 
 	$logger->log(LOG_INFO, "Response: " . $str . " (at " . gmdate("c") . " " . microtime() . ")");
 
+	if ($logger->request !== NULL)
+		$logger->request->write();
+
 	echo $str;
 	exit;
 }
